# -*- coding: utf-8 -*-
"""
:Authors: cykooz
:Date: 19.11.2015
"""
import re

import six
from six.moves.urllib.parse import parse_qsl, unquote_plus, urlparse


__all__ = (
    'Url',
    'Dict', 'D',
    'List', 'L',
    'AnyValue', 'ANY',
    'RegExpString', 'R',
)


class Url(object):
    """A url object that can be compared with other url objects
    without regard to the vagaries of encoding, escaping, and ordering
    of parameters in query strings.

        >>> url1 = Url('http://domain.com/container?limit=6&offset=0')
        >>> url2 = Url('http://domain.com/container?offset=0&limit=6')
        >>> url1 == url2
        True
        >>> url2 = Url('http://domain.com/container?limit=6')
        >>> url1 == url2
        False
        >>> url1 == 'http://domain.com/container?offset=0&limit=6'
        True
        >>> 'http://domain.com/container?offset=0&limit=6' == url1
        True
        >>> {'key': 'http://domain.com/container?offset=0&limit=6'} == {'key': url1}
        True
    """

    __slots__ = ('parts',)

    def __init__(self, url):
        parts = urlparse(url)
        _query = frozenset(parse_qsl(parts.query))
        _path = unquote_plus(parts.path)
        parts = parts._replace(query=_query, path=_path)
        self.parts = parts

    def __eq__(self, other):
        if not isinstance(other, Url):
            other = Url(other)
        return self.parts == other.parts

    def __hash__(self):
        return hash(self.parts)


class Dict(dict):
    """A dict object that can be compared with other dict object
    without regard to keys that did not presents in the ``Dict`` instance.

        >>> d1 = Dict(a=1, b='foo')
        >>> d2 = {'a': 1, 'b': 'foo', 'c': True}
        >>> d1 == d2
        True
        >>> d2 == d1
        True
        >>> d1 != d2
        False
        >>> d3 = {'a': 1, 'c': True}
        >>> d1 == d3
        False
        >>> d3 == d1
        False
        >>> d1 != d3
        True
        >>> Dict({'a': 1})
        Dict({'a': 1})
    """

    def __init__(self, *args, **kwargs):
        super(Dict, self).__init__(*args, **kwargs)

    def __eq__(self, other):
        for key, value in self.items():
            if key not in other:
                return False
            if value != other[key]:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'Dict(%s)' % super(Dict, self).__repr__()


class List(list):
    """A list object that can be compared with other list object
    without regard to extra items contains in the other list object.

        >>> l1 = List([1, 'foo'])
        >>> l2 = [1, 'foo', True]
        >>> l1 == l2
        True
        >>> l2 == l1
        True
        >>> l1 != l2
        False
        >>> l3 = [1, True]
        >>> l1 == l3
        False
        >>> l3 == l1
        False
        >>> l1 != l3
        True
        >>> l1 == [1]
        False
        >>> List([1, 'foo', True])
        List([1, 'foo', True])
        >>> List([Dict(), Dict()]) == [{'a': 1}, {'b': 2}]
        True
    """

    def __init__(self, *args, **kwargs):
        super(List, self).__init__(*args, **kwargs)

    def __eq__(self, other):
        if not isinstance(other, list):
            return False
        if len(self) > len(other):
            return False
        for v1, v2 in zip(self, other):
            if v1 != v2:
                return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return 'List(%s)' % super(List, self).__repr__()


class AnyValue(object):
    """Instance of this class is equal to any other values.

        >>> v = AnyValue()
        >>> v == 1
        True
        >>> 1 == v
        True
        >>> v != 1
        False
        >>> v == {'a': 1, 'b': 'foo'}
        True
        >>> v == [1, 2, 3, 'b']
        True
        >>> v == AnyValue()
        True
        >>> v
        <any value>
        >>> {v: 1}
        Traceback (most recent call last):
        ...
        TypeError: unhashable type: 'AnyValue'
        >>> [v, v, v] == [1, 2, 'foo']
        True
        >>> [v, v, 1] == [1, 2, 'foo']
        False
        >>> [v, v] == [1, 2, 'foo']
        False
        >>> {'a': v, 'b': 2} == {'a': 1, 'b': 2}
        True
    """

    __hash__ = None

    def __eq__(self, other):
        return True

    def __ne__(self, other):
        return False

    def __repr__(self):
        return '<any value>'


class RegExpString(object):
    """Instance of this class is equal to any other values if it is matched to give regexp pattern.

        >>> v = RegExpString('first.*')
        >>> v == 1
        False
        >>> 1 == v
        False
        >>> v != 1
        True
        >>> v == 'first class'
        True
        >>> 'first class' == v
        True
        >>> v != 'first class'
        False
        >>> v
        <RegExpString: first.*>
        >>> {v: 1}
        Traceback (most recent call last):
        ...
        TypeError: unhashable type: 'RegExpString'
        >>> [v, v, v] == [1, 2, 'first class']
        False
        >>> [v, v, v] == ['first class', 'first bus', 'first time']
        True
    """

    def __init__(self, pattern, flags=re.UNICODE):
        self.pattern = pattern
        self.re = re.compile(pattern, flags)

    __hash__ = None

    def __eq__(self, other):
        if isinstance(other, six.binary_type):
            other = other.decode('utf-8')
        elif not isinstance(other, six.text_type):
            other = six.text_type(other)
        return bool(self.re.match(other))

    def __ne__(self, other):
        return not self.__eq__(other)

    def __repr__(self):
        return '<RegExpString: %s>' % self.pattern


# Short aliases
ANY = AnyValue()
D = Dict
L = List
R = RegExpString



# -*- coding: utf-8 -*-
"""Calculates the current version number.

If possible, uses output of “git describe” modified to conform to the
visioning scheme that setuptools uses (see PEP 440).  Releases must be
labelled with annotated tags (signed tags are annotated) of the following
format:

   v<num>(.<num>)+[{a|b|c|rc}<num>(.<num>)*]

If “git describe” returns an error (likely because we're in an unpacked copy
of a release tarball, rather than a git working copy), or returns a tag that
does not match the above format, version is read from RELEASE-VERSION file.

To use this script, simply import it your setup.py file, and use the results
of getVersion() as your package version:

    import version
    setup(
        version=version.get_version(),
        .
        .
        .
    )

This will automatically update the RELEASE-VERSION file.  The RELEASE-VERSION
file should *not* be checked into git but it *should* be included in sdist
tarballs (as should version.py file).  To do this, run:

    echo include RELEASE-VERSION version.py >>MANIFEST.in
    echo RELEASE-VERSION >>.gitignore

With that setup, a new release can be labelled by simply invoking:

    git tag -a v1.0
"""
from __future__ import print_function

import re
import subprocess
import sys
from distutils.version import StrictVersion

__author__ = ('Douglas Creager <dcreager@dcreager.net>',
              'Michal Nazarewicz <mina86@mina86.com>',
              'Kirill Kuzminykh <cykooz@gmail.com')
__license__ = 'This file is placed into the public domain.'
__maintainer__ = 'Kirill Kuzminykh'
__email__ = 'cykooz@gmail.com'

__all__ = ('get_version',)


CHANGES_FILE = 'CHANGES.rst'
RELEASE_VERSION_FILE = 'RELEASE-VERSION'

# http://legacy.python.org/dev/peps/pep-0440/
_PEP440_SHORT_VERSION_RE = r'\d+(?:\.\d+)+(?:(?:[abc]|rc)\d+(?:\.\d+)*)?'
_PEP440_VERSION_RE = r'^%s(?:\.post\d+)?(?:\.dev\d+)?$' % _PEP440_SHORT_VERSION_RE
_GIT_DESCRIPTION_RE = r'^v(?P<ver>%s)-(?P<commits>\d+)-g(?P<sha>[\da-f]+)$' % _PEP440_SHORT_VERSION_RE


def read_git_version():
    try:
        proc = subprocess.Popen(('git', 'describe', '--long',
                                 '--match', 'v[0-9]*.*'),
                                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        data, _ = proc.communicate()
        if proc.returncode:
            return None, None
        ver = data.decode('utf-8').splitlines()[0].strip()
    except:
        return None, None

    if not ver:
        return None, None
    m = re.search(_GIT_DESCRIPTION_RE, ver)
    if not m:
        sys.stderr.write('version: git description (%s) is invalid, '
                         'ignoring\n' % ver)
        return None, None

    commits = int(m.group('commits'))
    ver = StrictVersion(m.group('ver'))
    if not commits:
        return ver, commits
    else:
        if len(ver.version) == 2:
            ver.version += (0,)
        ver.version = (ver.version[0], ver.version[1], ver.version[2] + commits)
        return ver, commits


def read_release_version():
    try:
        fd = open(RELEASE_VERSION_FILE)
        try:
            ver = fd.readline().strip()
        finally:
            fd.close()
        if not re.search(_PEP440_VERSION_RE, ver):
            sys.stderr.write('version: release version (%s) is invalid, '
                             'will use it anyway\n' % ver)
        return StrictVersion(ver)
    except:
        return None


def write_release_version(version):
    fd = open(RELEASE_VERSION_FILE, 'w')
    fd.write('%s\n' % version)
    fd.close()


def get_version():
    release_version = read_release_version()
    version = read_git_version()[0] or release_version
    if not version:
        print('ERROR: Cannot find the version number for "mountbit"')
        version = StrictVersion('0.0')
    if not release_version or version != release_version:
        write_release_version(version)
    return str(version)


def get_version_for_new_release():
    version, commits = read_git_version()
    if not version:
        raise ValueError('Cannot find the version number')
    return version, commits


def main():
    import argparse
    import datetime
    parser = argparse.ArgumentParser(description='Run periodic tasks for users')
    parser.add_argument('-u', dest='version',
                        help='replace title "Next release" in CHANGES.txt on given version number '
                             'and current date. You can use "auto" as version number for generate '
                             'version number automatically')
    args = parser.parse_args()
    if not args.version:
        print(get_version())
        return

    try:
        version, commits = get_version_for_new_release()
        if commits == 0:
            print(version)
            return
    except ValueError:
        if args.version == 'auto':
            raise
        version = args.version

    with open(CHANGES_FILE, 'rt') as f:
        changes = f.read()
    find = 'Next release\n============\n'
    if find in changes:
        if args.version == 'auto':
            # Increase the patch number to account for the additional commit.
            version.version = (version.version[0], version.version[1], version.version[2] + 1)
        else:
            version = StrictVersion(args.version)
        replace = '%s (%s)' % (version, datetime.date.today().strftime('%Y-%m-%d'))
        replace = '%s\n%s\n' % (replace, '=' * len(replace))
        new_changes = changes.replace(find, replace)
        if new_changes != changes:
            with open(CHANGES_FILE, 'wt') as f:
                f.write(new_changes)
            print(version)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
:Authors: cykooz
:Date: 13.05.2019
"""


def runtests():
    import sys
    import pytest
    from os import environ
    from os.path import dirname, join
    cfg_path = join(dirname(dirname(dirname(__file__))), 'setup.cfg')

    args = sys.argv[1:]
    if not args or args[0].startswith('-'):
        args += ['--pyargs', 'cykooz.testing']
    args = ['-c', cfg_path] + args
    environ['IS_TESTING'] = 'True'
    pytest.main(args)



# -*- coding: utf-8 -*-
"""
:Authors: cykooz
:Date: 25.06.2015
"""
import os
import sys

from setuptools import find_packages, setup


HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.append(HERE)

import version


README = open(os.path.join(HERE, 'README.rst'), 'rt').read()
CHANGES = open(os.path.join(HERE, 'CHANGES.rst'), 'rt').read()


def cli_cmd(app_name, command_name, func_name=None):
    func_name = func_name or command_name
    tpl = '{cmd} = cykooz.testing.{app}.commands.{cmd}:{func}.cli'
    return tpl.format(cmd=command_name, app=app_name, func=func_name)


setup(
    name='cykooz.testing',
    version=version.get_version(),
    description='Collection of helper utilities for testing.',
    long_description=README + '\n\n' + CHANGES,
    long_description_content_type='text/x-rst',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Testing',
        'License :: OSI Approved :: MIT License',
    ],
    keywords='',
    author='Cykooz',
    author_email='cykooz@gmail.com',
    url='https://github.com/Cykooz/cykooz.testing',
    package_dir={'': '.'},
    packages=find_packages(),
    include_package_data=True,
    package_data={},
    zip_safe=False,
    extras_require={
        'test': [
            'pytest',
        ],
    },
    install_requires=[
        'setuptools',
        'six',
    ],
    entry_points={
        'console_scripts':
            [
                'tests = cykooz.testing.runtests:runtests [test]',
            ]
    },
)
